/*
   libwrap.c  TCP Wrapper for IP v4/v6
   $Header: /home/fujiwara/.cvsroot/tcpd/rfc1413.c,v 1.9 1997/11/14 13:03:47 fujiwara Exp $
   tcp_wrappers_7.6 compatible package for IPv6+IPv4
   written by Kazunori Fujiwara <fujiwara@rcac.tdi.co.jp>
 */

#include <setjmp.h>
#include <unistd.h>
#include <signal.h>
#include <netdb.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#include <tcpd.h>

#define IDENT_PORT 113
#define	STRINGLEN  256

static int writes(int sock, char *buf)
{
	char *p;
	int i, len;

	p = buf;
	len = strlen(buf);

	while(len > 0) {
		if ((i = write(sock, p, len)) <= 0)
			return 0;
		len -= i;
		p += i;
	}
	return 1;
}

static int read1(int sock, char *buf, int len)
{
	char *p, *q;
	int i, left;

	p = buf;
	left = len - 1;
	while((i = read(sock, p, left)) > 0) {
		q = p;
		p += i;
		left -= i;
		*p = '\0';
		if (strchr(q, '\n') != NULL)
			break;
	}
	return i > 0;
}

static jmp_buf timeoutbuf;

static void timeout(int sig)
{
	longjmp(timeoutbuf, sig);
}

char *rfc1413(
	union tcpd_sock_union *local,
	union tcpd_sock_union *remote,
	 char *user, int userlen, int rfc1413_timeout)
{
	int rp, lp, salen, s;
	int nrp, nlp;
	char *p, *ret = NULL;
	union tcpd_sock_union sal, sar;
	char buf[STRINGLEN];
	char buf2[STRINGLEN];

	if (rfc1413_timeout <= 0)
		return NULL;

	user[0] = '\0';
	bzero(&sal, sizeof(sal));
	bzero(&sar, sizeof(sar));
	switch(remote->sa4.sin_family) {
	case AF_INET:
		sal.sa4.sin_family = AF_INET;
		sar.sa4.sin_family = AF_INET;
		sal.sa4.sin_addr = local->sa4.sin_addr;
		sar.sa4.sin_addr = remote->sa4.sin_addr;
		sar.sa4.sin_port = htons(IDENT_PORT);
		rp = ntohs(remote->sa4.sin_port);
		lp = ntohs(local->sa4.sin_port);
		salen = sizeof(sal.sa4);
		break;
#ifdef INET6
	case AF_INET6:
		sal.sa6.sin6_family = AF_INET;
		sar.sa6.sin6_family = AF_INET;
		sal.sa6.sin6_addr = local->sa6.sin6_addr;
		sar.sa6.sin6_addr = remote->sa6.sin6_addr;
		sar.sa6.sin6_port = htons(IDENT_PORT);
		rp = ntohs(remote->sa6.sin6_port);
		lp = ntohs(local->sa6.sin6_port);
		salen = sizeof(sal.sa6);
		break;
#endif
	default:
		return NULL;
	}
	snprintf(buf, sizeof(buf), "%u,%u\r\n", rp, lp);
	if ((s = socket(remote->sa4.sin_family, SOCK_STREAM, 0)) < 0)
		return NULL;
	if (setjmp(timeoutbuf) == 0) {
		signal(SIGALRM, timeout);
		alarm(rfc1413_timeout);
		if (bind(s, (struct sockaddr *)&sal, salen) == 0
		  && connect(s, (struct sockaddr *)&sar, salen) >= 0
		  && writes(s, buf)
		  && read1(s, buf, sizeof(buf))
		  && sscanf(buf, "%u , %u : USERID :%*[^:]:%255s",
			&nrp, &nlp, buf2) == 3
		  && nlp == lp && nrp == rp) {
			if ((p = strchr(buf2, '\r')) != NULL)
				*p = '\0';
			strncpy(user, buf2, userlen);
			ret = user;
		}
		alarm(0);
		close(s);
	} else {
		close(s);
	}
	signal(SIGALRM, SIG_IGN);

	return ret;
}

