/* /usr/include/tcpd.h */
/*
   tcp_wrappers_7.6 compatible package for IPv4 and IPv6
   written by Kazunori Fujiwara <fujiwara@rcac.tdi.co.jp>
 */

#define	TCPD_HOSTNAMELEN	257
#define	TCPD_STRINGLEN		64
#define	TCPD_STRINGLENL		257

#define TCPD_DEFAULT_RFC1413_TIMEOUT	30

/* should be struct sockaddr_storage */

union tcpd_sock_union {
	struct sockaddr_in sa4;
#ifdef INET6
	struct sockaddr_in6 sa6;
#endif
	unsigned char uu[136];
};

struct request_info {
	int sock; /* socket or file descriptor */
	union tcpd_sock_union src;
	union tcpd_sock_union dst;
	char src_name[TCPD_HOSTNAMELEN];
	char dst_name[TCPD_HOSTNAMELEN];
	char service[TCPD_STRINGLEN];
	char user[TCPD_STRINGLEN];
	char wrongdns;
	char error;
	char udp;
	char userresolved;
	int lineno;
	int refuse_wait;
	int rfc1413_timeout;
	char commands_default[TCPD_STRINGLENL];	/* optional commands:default */
	char commands[TCPD_STRINGLENL];	/* optional commands */
};

#define	RQ_END		0 /* */
#define	RQ_FILE		1 /* int: socket fd */
#define	RQ_DAEMON	2 /* char *: service name */

#define	RQ_USER		3 /* set user (for test) */
#define	RQ_SERVER_SIN	4 /* ignore : use RQ_SERVER_ADDR */
#define	RQ_CLIENT_SIN	5 /* ignore : use RQ_CLIENT ADDR */
#define	RQ_SERVER_ADDR	6 /* set server addr (string) */
#define	RQ_CLIENT_ADDR	7 /* set client addr (string) */
#define	RQ_SERVER_NAME	8 /* set server FDQN */
#define	RQ_CLIENT_NAME	9 /* set client FDQN */
#define RQ_SETTABLE	10 /* set another access table */

void request_init(struct request_info *, ...);
void fromhost(struct request_info *);
int hosts_access(struct request_info *);
void tcpd_satostr(char *str, int len, union tcpd_sock_union *sa);
void refuse(struct request_info *);
char *rfc1413(union tcpd_sock_union *local, union tcpd_sock_union *remote,  char *user, int userlen, int rfc1413_timeout);
int tcpd_exec_option(struct request_info *req);

#define TCPD_ALLOW 1
#define TCPD_DENY  0
#define TCPD_ERROR -1

extern int allow_severity;
extern int deny_severity;
